/* preferences-dialog.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

/* @todo set_smart_home_end */
public class Valide.PreferencesDialog : AbstractPreferencesDialog
{
  [CCode (instance_pos = -1)]
  protected void update_param (Gtk.Widget sender)
  {
    try
    {
      Value val;
      string key = Utils.str_replace (sender.get_name (), "_", "-");
      string group = "GtkSourceView";

      if (key == "right-margin-position" || key == "indent-width" || key == "tab-width")
      {
        Gtk.SpinButton btn = (Gtk.SpinButton)sender;

        val = Value (typeof (int));
        val.set_int (btn.get_value_as_int ());
        ConfigManager.get_instance ().set_integer (group, key, val.get_int ());
      }
      else
      {
        Gtk.ToggleButton btn = (Gtk.ToggleButton)sender;

        val = Value (typeof (bool));
        val.set_boolean (btn.get_active ());
        ConfigManager.get_instance ().set_boolean (group, key, val.get_boolean ());
      }

      /* Update GtkSourceView display */
      if (group == "GtkSourceView")
      {
        foreach (iDocument document in DocumentManager.get_instance ().documents)
        {
          if (document is AbstractTextDocument)
          {
            AbstractTextDocument atd = (AbstractTextDocument)document;

            atd.view1.set_property (key, val);
            atd.view2.set_property (key, val);
          }
        }
      }
    }
    catch (Error e)
    {
      debug (e.message);
    }
  }

  protected void set_param () throws Error
  {
    KeyFile key_file = new KeyFile ();
    key_file.load_from_file (ConfigManager.get_instance ().config_file, KeyFileFlags.NONE);

    string[] groups = key_file.get_groups ();

    foreach (string g in groups)
    {
      try
      {
        string[] keys = key_file.get_keys (g);

        foreach (string k in keys)
        {
          string name = Utils.str_replace (k, "-", "_");
          
          if (k == "right-margin-position" || k == "indent-width" || k == "tab-width")
          {
            Gtk.SpinButton btn = (Gtk.SpinButton)this.xml.get_object (name);

            if (btn != null)
            {
              btn.set_value ((double)ConfigManager.get_instance ().get_integer (g, k));
            }
          }
          else
          {
            Gtk.ToggleButton btn = (Gtk.ToggleButton)this.xml.get_object (name);

            if (btn != null)
            {
              btn.set_active (ConfigManager.get_instance ().get_boolean (g, k));
            }
          }
        }
      }
      catch (Error e)
      {
        debug (e.message);
      }
    }
  }

  construct
  {
    Gtk.Adjustment adj = new Gtk.Adjustment (0.0, 0.0, 100.0, 1.0, 10.0, 10.0);
    this.widgets.indent_width.set_adjustment (adj);
    adj = new Gtk.Adjustment (0.0, 0.0, 100.0, 1.0, 10.0, 10.0);
    this.widgets.tab_width.set_adjustment (adj);
    adj = new Gtk.Adjustment (0.0, 0.0, 100.0, 1.0, 10.0, 10.0);
    this.widgets.right_margin_position.set_adjustment (adj);

    try
    {
      this.set_param ();
    }
    catch (Error e)
    {
      debug (e.message);
    }

    this.connect_signals ("valide_preferences_dialog_");

    this.widgets.notebook.append_page (PluginManager.get_instance ().preferences (),
                                       new Gtk.Label (_("Plugins")));
  }

  public int run ()
  {
    this.widgets.dialog.show_all ();
    return this.widgets.dialog.run ();
  }

  public new void destroy ()
  {
    this.widgets.dialog.destroy ();
  }
}

