/* searching.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public class Valide.Searching : Gtk.VBox
{
  private Gtk.CheckButton check_casse;
  private Gtk.CheckButton check_word;
  private Gtk.Entry entry;
  private bool reverse = false;

  public AbstractTextDocument document
  {
    get;
    construct;
  }

  public string text
  {
    get { return this.entry.get_text (); }
    set { this.entry.set_text (value); }
  }

  public bool casse
  {
    get { return this.check_casse.get_active (); }
    set { this.check_casse.active = value; }
  }

  public bool word
  {
    get { return this.check_word.get_active (); }
    set { this.check_word.active = value; }
  }

  private void search ()
  {
    Gtk.SourceSearchFlags flags = Gtk.SourceSearchFlags.TEXT_ONLY;
    Gtk.SourceBuffer buffer = this.document.buffer;
    Gtk.TextIter iter;

    buffer.get_start_iter (out iter);
    Gtk.TextMark mark = buffer.get_mark ("search-start");
    if (mark == null)
    {
      mark = buffer.create_mark ("search-start", iter, false);
    }
    if (!this.casse)
    {
      flags |= Gtk.SourceSearchFlags.CASE_INSENSITIVE;
    }

    int count = 0;
    bool found = false;
    Gtk.TextIter start = Gtk.TextIter ();
    Gtk.TextIter end = Gtk.TextIter ();

    while (count < 2)
    {
      buffer.get_iter_at_mark (out iter, mark);
      if (!this.reverse)
      {
        found = Gtk.source_iter_forward_search (iter, this.text, flags, out start, out end, null);
      }
      else
      {
        if (count == 0)
        {
          iter.backward_word_start ();
        }
        found = Gtk.source_iter_backward_search (iter, this.text, flags, out start, out end, null);
      }
      if (found && this.word)
      {
        found = start.starts_word () &&  end.ends_word ();
      }
      if (!found)
      {
        if (!this.reverse)
        {
          buffer.get_start_iter (out iter);
        }
        else
        {
          buffer.get_end_iter (out iter);
        }
        buffer.create_mark ("search-start", iter, false);
        count++;
      }
      else
      {
        break;
      }
    }

    Gdk.Color color;

    if (found)
    {
      buffer.move_mark (mark, end);
      this.document.text_view.scroll_to_mark (mark, 0.4, false, 0, 0);
      buffer.select_range (start, end);
  		Gdk.Color.parse ("white", out color);
    }
    else
    {
      buffer.get_start_iter (out iter);
      buffer.create_mark ("search-start", iter, false);
   		Gdk.Color.parse ("#FFBFBF", out color);
    }
    this.entry.modify_base (Gtk.StateType.NORMAL, color);
  }

  public Searching (AbstractTextDocument document)
  {
    this.document = document;
  }

  construct
  {
    this.pack_start (new Gtk.HSeparator (), false, true, 0);

    Gtk.HBox hbox = new Gtk.HBox (false, 0);
    this.pack_start (hbox, true, true, 0);

    Gtk.Button button = Utils.create_small_button_from_stock (Gtk.STOCK_CLOSE);
    button.clicked += () => { this.hide (); };
    hbox.pack_start (button, false, true, 10);

    hbox.pack_start (new Gtk.Label (_("Find: ")), false, false, 0);
    this.entry = new Gtk.Entry ();
    this.entry.activate += () => {
      this.find_next ();
    };
    this.entry.key_press_event += (s, e) => {
      Gdk.EventKey ev = (Gdk.EventKey)e;

      if (ev.keyval == Gdk.KeySyms.Escape)
      {
        this.hide ();
        this.document.grab_focus ();
      }
      return false;
    };
    hbox.pack_start (this.entry, false, true, 10);

    button = new Gtk.Button.from_stock (Gtk.STOCK_GO_BACK);
    button.set_relief (Gtk.ReliefStyle.NONE);
    button.clicked += () => {
      this.find_prev ();
    };
    hbox.pack_start (button, false, false, 10);

    button = new Gtk.Button.from_stock (Gtk.STOCK_GO_FORWARD);
    button.set_relief (Gtk.ReliefStyle.NONE);
    button.clicked += () => {
      this.find_next ();
    };
    hbox.pack_start (button, false, false, 10);

    this.check_casse = new Gtk.CheckButton.with_label (_("Match case"));
    hbox.pack_start (this.check_casse, false, false, 10);

    this.check_word = new Gtk.CheckButton.with_label (_("Match entire word only"));
    hbox.pack_start (this.check_word, false, false, 10);
  }

  public void find (bool show_dialog = true)
  {
    string t = this.document.selected_text;

    if (show_dialog || (t == "" && this.text == ""))
    {
      if (t != "")
      {
        this.text = t;
      }

      Gtk.TextBuffer buffer = this.document.buffer;
      Gtk.TextIter iter;

      buffer.get_start_iter (out iter);
      buffer.create_mark ("search-start", iter, false);
      this.show ();
    }
    else
    {
      this.search ();
    }
  }

  public void find_next ()
  {
    this.reverse = false;
    this.find (false);
  }

  public void find_prev ()
  {
    this.reverse = true;
    this.find (false);
  }

  public new void show ()
  {
    base.show_all ();
    this.entry.grab_focus ();
    if (this.text != "")
    {
      this.entry.select_region (0, (int)this.text.size ());
    }
  }
}

