/* start-page.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

private class Valide.LinkLabel : Gtk.EventBox
{
  public signal void open_uri (string uri);

  public string uri
  {
    get;
    construct;
  }

  public LinkLabel (string uri, string title)
  {
    Gtk.Label label = new Gtk.Label ("");
    label.set_alignment (0, 0.5f);
    label.set_markup ("<span color=\"blue\"><u>%s</u></span>".printf (title));
    this.add (label);
    this.uri = uri;
    this.button_press_event += () => {
      this.open_uri (this.uri);
    };
    this.enter_notify_event += () => {
      Gdk.Cursor cursor = new Gdk.Cursor (Gdk.CursorType.HAND2);
      this.window.set_cursor (cursor);
      return false;
    };
    this.leave_notify_event += () => {
      Gdk.Cursor cursor = new Gdk.Cursor (Gdk.CursorType.ARROW);
      this.window.set_cursor (cursor);
      return false;
    };
  }
}

public class Valide.StartPage : Gtk.Table, iDocument
{
  private uint status_context;

  private delegate void OpenFunc (Gtk.LinkButton sender);

  public new DocumentState state
  {
    get
    {
      return DocumentState.INFO;
    }
    set
    {
    }
  }

  public bool is_save
  {
    get
    {
      return true;
    }
    set
    {
    }
  }

  public new string path
  {
    owned get
    {
      return "";
    }
    set
    {
    }
  }

  public string title
  {
    owned get
    {
      return this.filename;
    }
  }

  public string filename
  {
    owned get
    {
      return _("Start page");
    }
  }

  [CCode (instance_pos = -1)]
  private void open_file (LinkLabel sender, string uri)
  {
    DocumentManager.get_instance ().create (uri);
    this.close ();
  }

  [CCode (instance_pos = -1)]
  private void open_project (LinkLabel sender, string uri)
  {
    if (ProjectManager.get_instance ().open (uri))
    {
      this.close ();
    }
  }

  private Gtk.Frame create_frame (string title, List<Gtk.RecentInfo>? items,
                                  Callback callback)
  {
    Gtk.Frame frame = new Gtk.Frame (title);

    Gtk.Alignment alignment = new Gtk.Alignment (0, 0, 0, 0);
    frame.add (alignment);

    Gtk.VBox vbox = new Gtk.VBox (false, 5);
    vbox.set_border_width (5);
    alignment.add (vbox);

    if (items != null)
    {
      foreach (Gtk.RecentInfo project in items)
      {
        string uri = project.get_uri ();
        string label = Path.get_basename (uri);
        LinkLabel link_label = new LinkLabel (uri, label);
        link_label.enter_notify_event += (s) => {
          string status_message = _("Open %s").printf (s.uri);
          Statusbar.get_instance ().flash_message (this.status_context,
                                                   status_message);
          return false;
        };
  
        Signal.connect (link_label, "open_uri", callback, this);
        vbox.pack_start (link_label, false, false, 0);
      }
    }
    else
    {
      Gtk.Label label = new Gtk.Label (_("No recent item"));
      vbox.pack_start (label, false, false, 0);
    }
    return frame;
  }

  construct
  {
    this.resize (7, 2);

    Gtk.Image logo = new Gtk.Image.from_file (Path.build_filename (Config.PACKAGE_PIXMAPS_DIR, "logo.png"));
    this.attach (logo, 0, 2, 0, 1, Gtk.AttachOptions.EXPAND,
                 Gtk.AttachOptions.EXPAND, 0, 0);

    Gtk.Frame frame = new Gtk.Frame ("");
    this.attach (frame, 0, 1, 1, 5,
                 Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL,
                 Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL, 2, 0);

    Gtk.VBox vbox = new Gtk.VBox (false, 5);
    vbox.set_border_width (5);
    frame.add (vbox);

    Gtk.HBox hbox = new Gtk.HBox (false, 0);
    vbox.pack_start (hbox, false, false, 0);
    Gtk.Image image = new Gtk.Image.from_stock (Gtk.STOCK_NEW, Gtk.IconSize.BUTTON);
    hbox.pack_start (image, false, false, 2);
    LinkLabel link_label = new LinkLabel ("", _("New project"));
    link_label.open_uri += () => {
      if (ProjectManager.get_instance ().create ())
      {
        this.close ();
      }
    };
    hbox.pack_start (link_label, false, false, 2);

    hbox = new Gtk.HBox (false, 0);
    vbox.pack_start (hbox, false, false, 0);
    image = new Gtk.Image.from_stock (Gtk.STOCK_OPEN, Gtk.IconSize.BUTTON);
    hbox.pack_start (image, false, false, 2);
    link_label = new LinkLabel ("", _("Open project"));
    link_label.open_uri += () => {
      if (ProjectManager.get_instance ().open ())
      {
        this.close ();
      }
    };
    hbox.pack_start (link_label, false, false, 2);

    vbox = new Gtk.VBox (true, 0);
    this.attach (vbox, 0, 1, 1, 3, Gtk.AttachOptions.EXPAND,
                 Gtk.AttachOptions.EXPAND, 0, 0);

    frame = this.create_frame (_("Recent projects"),
                               RecentManagerUI.get_instance ().recent_projects,
                               (Callback)this.open_project);
    this.attach (frame, 1, 2, 1, 3,
                 Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL,
                 Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL, 2, 0);

    frame = this.create_frame (_("Recent files"), 
                               RecentManagerUI.get_instance ().recent_files,
                               (Callback)this.open_file);
    this.attach (frame, 1, 2, 3, 5,
                 Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL,
                 Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL, 2, 0);

    Gtk.HSeparator sep = new Gtk.HSeparator ();
    this.attach (sep, 0, 2, 5, 6, Gtk.AttachOptions.FILL,
                 Gtk.AttachOptions.FILL, 0, 5);

    Gtk.Label label = new Gtk.Label (Config.COPYRIGHT);
    this.attach (label, 0, 2, 6, 7, Gtk.AttachOptions.FILL,
                 Gtk.AttachOptions.FILL, 0, 0);

    this.status_context = Statusbar.get_instance ()
                            .get_context_id ("valide_link_label");
  }

  public void open (string uri) throws Error
  {
  }

  public void save ()
  {
  }

  public void save_as ()
  {
  }

  public bool close ()
  {
    this.closed ();
    return true;
  }

  public void redo ()
  {
  }

  public void search ()
  {
  }

  public void find_next ()
  {
  }

  public void find_prev ()
  {
  }

  public bool goto_line (int line = -1)
  {
    return true;
  }

  public Gdk.Pixbuf get_icon ()
  {
    return Utils.get_pixbuf_for_stock (Gtk.STOCK_INFO, Gtk.IconSize.MENU);
  }
}

