/* template.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

public class Valide.Template : Object
{
  private List<string> _files;

  public string name { get; private set; }
  public string desc { get; private set; }
  public string path { get; private set; }
  public string filename { get; construct; }
  public Gdk.Pixbuf icon { get; private set; }

  public List<string> files
  {
    get
    {
      return this._files.first ();
    }
    private set
    {
      this.files = value;
    }
  }

  public Template (string filename)
  {
    this.filename = filename;
  }

  construct
  {
    try
    {
      Xml.Doc* xml = Xml.Parser.parse_file (this.filename);
      if (xml != null)
      {
        this.path = Path.get_dirname (this.filename);
        Xml.XPathContext xpath = new Xml.XPathContext (xml);
        Xml.XPathObject* xpath_obj = xpath.eval ("/template/files/file/text ()");
        for (int i = 0; i < xpath_obj->nodesetval->length (); i++)
        {
          this._files.append (Path.build_filename (this.path,
                                                   xpath_obj->nodesetval->item (i)->content,
                                                   null));
        }

        xpath_obj = xpath.eval ("/template/name/text ()");
        this.name = xpath_obj->nodesetval->item (0)->content;

        xpath_obj = xpath.eval ("/template/description/text ()");
        this.desc = xpath_obj->nodesetval->item (0)->content;

        this.icon = new Gdk.Pixbuf.from_file (Path.build_filename (this.path, "/icon.png"));
      }
    }
    catch (Error e)
    {
      debug (e.message);
    }
  }
}

