/* abstract-config-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __LIBVALIDE_ABSTRACT_CONFIG_MANAGER_H__
#define __LIBVALIDE_ABSTRACT_CONFIG_MANAGER_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>

G_BEGIN_DECLS


#define VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER (valide_abstract_config_manager_get_type ())
#define VALIDE_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManager))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))

typedef struct _ValideAbstractConfigManager ValideAbstractConfigManager;
typedef struct _ValideAbstractConfigManagerClass ValideAbstractConfigManagerClass;
typedef struct _ValideAbstractConfigManagerPrivate ValideAbstractConfigManagerPrivate;

/**
 * Abstract class for manage configation files.
 */
struct _ValideAbstractConfigManager {
	GObject parent_instance;
	ValideAbstractConfigManagerPrivate * priv;
};

struct _ValideAbstractConfigManagerClass {
	GObjectClass parent_class;
	void (*preferences) (ValideAbstractConfigManager* self);
	void (*first_run) (ValideAbstractConfigManager* self);
	char* (*get_config_file) (ValideAbstractConfigManager* self);
};


void valide_abstract_config_manager_preferences (ValideAbstractConfigManager* self);
void valide_abstract_config_manager_first_run (ValideAbstractConfigManager* self);
char* valide_abstract_config_manager_create_file (ValideAbstractConfigManager* self, const char* filename, const char* contents);
gint valide_abstract_config_manager_get_integer (ValideAbstractConfigManager* self, const char* group, const char* key);
char* valide_abstract_config_manager_get_string (ValideAbstractConfigManager* self, const char* group, const char* key);
gboolean valide_abstract_config_manager_get_boolean (ValideAbstractConfigManager* self, const char* group, const char* key);
void valide_abstract_config_manager_set_integer (ValideAbstractConfigManager* self, const char* group, const char* key, gint val);
void valide_abstract_config_manager_set_string (ValideAbstractConfigManager* self, const char* group, const char* key, const char* val);
void valide_abstract_config_manager_set_boolean (ValideAbstractConfigManager* self, const char* group, const char* key, gboolean val);
char* valide_abstract_config_manager_get_config_file (ValideAbstractConfigManager* self);
char* valide_abstract_config_manager_get_home (ValideAbstractConfigManager* self);
char* valide_abstract_config_manager_get_tmp_dir (ValideAbstractConfigManager* self);
GType valide_abstract_config_manager_get_type (void);


G_END_DECLS

#endif
