/* document-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __LIBVALIDE_DOCUMENT_MANAGER_H__
#define __LIBVALIDE_DOCUMENT_MANAGER_H__

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <libvalide/ui/abstract-close-dialog.h>
#include <libvalide/idocument.h>
#include <libvalide/abstract-text-document.h>

G_BEGIN_DECLS


#define VALIDE_TYPE_CLOSE_DIALOG (valide_close_dialog_get_type ())
#define VALIDE_CLOSE_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_CLOSE_DIALOG, ValideCloseDialog))
#define VALIDE_CLOSE_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_CLOSE_DIALOG, ValideCloseDialogClass))
#define VALIDE_IS_CLOSE_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_CLOSE_DIALOG))
#define VALIDE_IS_CLOSE_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_CLOSE_DIALOG))
#define VALIDE_CLOSE_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_CLOSE_DIALOG, ValideCloseDialogClass))

typedef struct _ValideCloseDialog ValideCloseDialog;
typedef struct _ValideCloseDialogClass ValideCloseDialogClass;
typedef struct _ValideCloseDialogPrivate ValideCloseDialogPrivate;

#define VALIDE_TYPE_DOCUMENT_MANAGER (valide_document_manager_get_type ())
#define VALIDE_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManager))
#define VALIDE_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))
#define VALIDE_IS_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_IS_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_DOCUMENT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))

typedef struct _ValideDocumentManager ValideDocumentManager;
typedef struct _ValideDocumentManagerClass ValideDocumentManagerClass;
typedef struct _ValideDocumentManagerPrivate ValideDocumentManagerPrivate;

typedef enum  {
	VALIDE_DOCUMENT_ERROR_UNKNOW_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_BAD_URI,
	VALIDE_DOCUMENT_ERROR_NO_DOCUMENT
} ValideDocumentError;
#define VALIDE_DOCUMENT_ERROR valide_document_error_quark ()
struct _ValideCloseDialog {
	ValideAbstractCloseDialog parent_instance;
	ValideCloseDialogPrivate * priv;
};

struct _ValideCloseDialogClass {
	ValideAbstractCloseDialogClass parent_class;
};

struct _ValideDocumentManager {
	GtkNotebook parent_instance;
	ValideDocumentManagerPrivate * priv;
};

struct _ValideDocumentManagerClass {
	GtkNotebookClass parent_class;
	void (*setup_ui) (ValideDocumentManager* self);
	ValideAbstractTextDocument* (*create_document) (ValideDocumentManager* self, const char* filename, GError** error);
};


GQuark valide_document_error_quark (void);
ValideCloseDialog* valide_close_dialog_construct (GType object_type, GList* documents);
ValideCloseDialog* valide_close_dialog_new (GList* documents);
guint valide_close_dialog_run (ValideCloseDialog* self);
void valide_close_dialog_destroy (ValideCloseDialog* self);
GList* valide_close_dialog_get_selected (ValideCloseDialog* self);
GType valide_close_dialog_get_type (void);
void valide_document_manager_setup_ui (ValideDocumentManager* self);
ValideAbstractTextDocument* valide_document_manager_create_document (ValideDocumentManager* self, const char* filename, GError** error);
ValideDocumentManager* valide_document_manager_get_instance (void);
ValideiDocument* valide_document_manager_create (ValideDocumentManager* self, const char* filename, GError** error);
void valide_document_manager_open (ValideDocumentManager* self, GError** error);
gboolean valide_document_manager_close_all (ValideDocumentManager* self);
gint valide_document_manager_append_page (ValideDocumentManager* self, ValideiDocument* child);
void valide_document_manager_remove_page (ValideDocumentManager* self, ValideiDocument* child);
void valide_document_manager_show_start_page (ValideDocumentManager* self);
void valide_document_manager_hide_start_page (ValideDocumentManager* self);
gboolean valide_document_manager_is_open (ValideDocumentManager* self, const char* filename, gint* pos);
ValideiDocument* valide_document_manager_get_current (ValideDocumentManager* self);
void valide_document_manager_set_current (ValideDocumentManager* self, ValideiDocument* value);
GList* valide_document_manager_get_documents (ValideDocumentManager* self);
GType valide_document_manager_get_type (void);


G_END_DECLS

#endif
