/* idocument.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __LIBVALIDE_IDOCUMENT_H__
#define __LIBVALIDE_IDOCUMENT_H__

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>

G_BEGIN_DECLS


#define VALIDE_TYPE_DOCUMENT_STATE (valide_document_state_get_type ())

#define VALIDE_TYPE_IDOCUMENT (valide_idocument_get_type ())
#define VALIDE_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocument))
#define VALIDE_IS_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_IDOCUMENT))
#define VALIDE_IDOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocumentIface))

typedef struct _ValideiDocument ValideiDocument;
typedef struct _ValideiDocumentIface ValideiDocumentIface;

typedef enum  {
	VALIDE_DOCUMENT_STATE_NORMAL,
	VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED,
	VALIDE_DOCUMENT_STATE_INFO
} ValideDocumentState;

struct _ValideiDocumentIface {
	GTypeInterface parent_iface;
	void (*open) (ValideiDocument* self, const char* uri, GError** error);
	void (*save) (ValideiDocument* self);
	void (*save_as) (ValideiDocument* self);
	gboolean (*close) (ValideiDocument* self);
	void (*redo) (ValideiDocument* self);
	void (*search) (ValideiDocument* self);
	void (*find_next) (ValideiDocument* self);
	void (*find_prev) (ValideiDocument* self);
	gboolean (*goto_line) (ValideiDocument* self, gint line);
	GdkPixbuf* (*get_icon) (ValideiDocument* self);
	ValideDocumentState (*get_state) (ValideiDocument* self);
	void (*set_state) (ValideiDocument* self, ValideDocumentState value);
	gboolean (*get_is_save) (ValideiDocument* self);
	void (*set_is_save) (ValideiDocument* self, gboolean value);
	char* (*get_path) (ValideiDocument* self);
	void (*set_path) (ValideiDocument* self, const char* value);
	char* (*get_title) (ValideiDocument* self);
	char* (*get_filename) (ValideiDocument* self);
};


GType valide_document_state_get_type (void);
void valide_idocument_open (ValideiDocument* self, const char* uri, GError** error);
void valide_idocument_save (ValideiDocument* self);
void valide_idocument_save_as (ValideiDocument* self);
gboolean valide_idocument_close (ValideiDocument* self);
void valide_idocument_redo (ValideiDocument* self);
void valide_idocument_search (ValideiDocument* self);
void valide_idocument_find_next (ValideiDocument* self);
void valide_idocument_find_prev (ValideiDocument* self);
gboolean valide_idocument_goto_line (ValideiDocument* self, gint line);
GdkPixbuf* valide_idocument_get_icon (ValideiDocument* self);
ValideDocumentState valide_idocument_get_state (ValideiDocument* self);
void valide_idocument_set_state (ValideiDocument* self, ValideDocumentState value);
gboolean valide_idocument_get_is_save (ValideiDocument* self);
void valide_idocument_set_is_save (ValideiDocument* self, gboolean value);
char* valide_idocument_get_path (ValideiDocument* self);
void valide_idocument_set_path (ValideiDocument* self, const char* value);
char* valide_idocument_get_title (ValideiDocument* self);
char* valide_idocument_get_filename (ValideiDocument* self);
GType valide_idocument_get_type (void);


G_END_DECLS

#endif
