/* panel.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __LIBVALIDE_PANEL_H__
#define __LIBVALIDE_PANEL_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

G_BEGIN_DECLS


#define VALIDE_TYPE_PANEL_ITEM (valide_panel_item_get_type ())
#define VALIDE_PANEL_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PANEL_ITEM, ValidePanelItem))
#define VALIDE_PANEL_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PANEL_ITEM, ValidePanelItemClass))
#define VALIDE_IS_PANEL_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PANEL_ITEM))
#define VALIDE_IS_PANEL_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PANEL_ITEM))
#define VALIDE_PANEL_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PANEL_ITEM, ValidePanelItemClass))

typedef struct _ValidePanelItem ValidePanelItem;
typedef struct _ValidePanelItemClass ValidePanelItemClass;
typedef struct _ValidePanelItemPrivate ValidePanelItemPrivate;
typedef struct _ValideParamSpecPanelItem ValideParamSpecPanelItem;

#define VALIDE_TYPE_PANEL (valide_panel_get_type ())
#define VALIDE_PANEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PANEL, ValidePanel))
#define VALIDE_PANEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PANEL, ValidePanelClass))
#define VALIDE_IS_PANEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PANEL))
#define VALIDE_IS_PANEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PANEL))
#define VALIDE_PANEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PANEL, ValidePanelClass))

typedef struct _ValidePanel ValidePanel;
typedef struct _ValidePanelClass ValidePanelClass;
typedef struct _ValidePanelPrivate ValidePanelPrivate;

/*
 * This code is re-writing in the vala language of GeditPanel
 *
 * Copyright (C) 2005 - Paolo Maggi 
 */
struct _ValidePanelItem {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValidePanelItemPrivate * priv;
	char* name;
	GtkImage* icon;
};

struct _ValidePanelItemClass {
	GTypeClass parent_class;
	void (*finalize) (ValidePanelItem *self);
};

struct _ValideParamSpecPanelItem {
	GParamSpec parent_instance;
};

struct _ValidePanel {
	GtkVBox parent_instance;
	ValidePanelPrivate * priv;
};

struct _ValidePanelClass {
	GtkVBoxClass parent_class;
	void (*close) (ValidePanel* self);
};


ValidePanelItem* valide_panel_item_construct (GType object_type);
ValidePanelItem* valide_panel_item_new (void);
GParamSpec* valide_param_spec_panel_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer valide_value_get_panel_item (const GValue* value);
void valide_value_set_panel_item (GValue* value, gpointer v_object);
GType valide_panel_item_get_type (void);
gpointer valide_panel_item_ref (gpointer instance);
void valide_panel_item_unref (gpointer instance);
ValidePanel* valide_panel_construct (GType object_type, GtkOrientation orientation);
ValidePanel* valide_panel_new (GtkOrientation orientation);
void valide_panel_add_item (ValidePanel* self, GtkWidget* item, const char* name, GtkImage* image);
void valide_panel_add_item_with_stock_icon (ValidePanel* self, GtkWidget* item, const char* name, const char* stock_id);
void valide_panel_close (ValidePanel* self);
gboolean valide_panel_remove_item (ValidePanel* self, GtkWidget* item);
GtkOrientation valide_panel_get_orientation (ValidePanel* self);
GType valide_panel_get_type (void);


G_END_DECLS

#endif
