/* result.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *   Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __PLUGINS_COMPLETION_RESULT_H__
#define __PLUGINS_COMPLETION_RESULT_H__

#include <glib.h>
#include <glib-object.h>
#include <gee/list.h>
#include <stdlib.h>
#include <string.h>

G_BEGIN_DECLS


#define COMPLETION_TYPE_RESULT (completion_result_get_type ())
#define COMPLETION_RESULT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), COMPLETION_TYPE_RESULT, CompletionResult))
#define COMPLETION_RESULT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), COMPLETION_TYPE_RESULT, CompletionResultClass))
#define COMPLETION_IS_RESULT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COMPLETION_TYPE_RESULT))
#define COMPLETION_IS_RESULT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), COMPLETION_TYPE_RESULT))
#define COMPLETION_RESULT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), COMPLETION_TYPE_RESULT, CompletionResultClass))

typedef struct _CompletionResult CompletionResult;
typedef struct _CompletionResultClass CompletionResultClass;
typedef struct _CompletionResultPrivate CompletionResultPrivate;

/*
 *  Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 */
struct _CompletionResult {
	GObject parent_instance;
	CompletionResultPrivate * priv;
	GeeList* properties;
	GeeList* classes;
	GeeList* interfaces;
	GeeList* structs;
	GeeList* methods;
	GeeList* fields;
	GeeList* signals;
	GeeList* others;
	GeeList* namespaces;
	GeeList* enums;
	GeeList* constants;
};

struct _CompletionResultClass {
	GObjectClass parent_class;
};


gboolean completion_result_namespaces_contain (CompletionResult* self, const char* name);
gboolean completion_result_classes_contain (CompletionResult* self, const char* name);
gboolean completion_result_interfaces_contain (CompletionResult* self, const char* name);
gboolean completion_result_structs_contain (CompletionResult* self, const char* name);
gboolean completion_result_constants_contain (CompletionResult* self, const char* name);
gboolean completion_result_enums_contain (CompletionResult* self, const char* name);
CompletionResult* completion_result_construct (GType object_type);
CompletionResult* completion_result_new (void);
gboolean completion_result_get_is_empty (CompletionResult* self);
gint completion_result_get_count (CompletionResult* self);
GType completion_result_get_type (void);


G_END_DECLS

#endif
