/* popup.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __PLUGINS_COMPLETION_SOURCE_COMPLETION_POPUP_H__
#define __PLUGINS_COMPLETION_SOURCE_COMPLETION_POPUP_H__

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>
#include <plugins/completion/source-completion/proposal.h>

G_BEGIN_DECLS


#define SOURCE_COMPLETION_TYPE_POPUP (source_completion_popup_get_type ())
#define SOURCE_COMPLETION_POPUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SOURCE_COMPLETION_TYPE_POPUP, SourceCompletionPopup))
#define SOURCE_COMPLETION_POPUP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SOURCE_COMPLETION_TYPE_POPUP, SourceCompletionPopupClass))
#define SOURCE_COMPLETION_IS_POPUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SOURCE_COMPLETION_TYPE_POPUP))
#define SOURCE_COMPLETION_IS_POPUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SOURCE_COMPLETION_TYPE_POPUP))
#define SOURCE_COMPLETION_POPUP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SOURCE_COMPLETION_TYPE_POPUP, SourceCompletionPopupClass))

typedef struct _SourceCompletionPopup SourceCompletionPopup;
typedef struct _SourceCompletionPopupClass SourceCompletionPopupClass;
typedef struct _SourceCompletionPopupPrivate SourceCompletionPopupPrivate;

#define SOURCE_COMPLETION_POPUP_TYPE_POSITION_TYPE (source_completion_popup_position_type_get_type ())

#define SOURCE_COMPLETION_POPUP_TYPE_FILTER_TYPE (source_completion_popup_filter_type_get_type ())

#define SOURCE_COMPLETION_POPUP_TYPE_KEYS_TYPE (source_completion_popup_keys_type_get_type ())

/*
 * This code is re-writing in the vala language of GtkSourceCompletion
 *
 * Copyright (C) 2007 - Chuchiperriman
 *
 * rev 163
 */
struct _SourceCompletionPopup {
	GtkWindow parent_instance;
	SourceCompletionPopupPrivate * priv;
};

struct _SourceCompletionPopupClass {
	GtkWindowClass parent_class;
};

typedef enum  {
	SOURCE_COMPLETION_POPUP_POSITION_TYPE_CURSOR,
	SOURCE_COMPLETION_POPUP_POSITION_TYPE_CENTER_SCREEN,
	SOURCE_COMPLETION_POPUP_POSITION_TYPE_CENTER_PARENT
} SourceCompletionPopupPositionType;

typedef enum  {
	SOURCE_COMPLETION_POPUP_FILTER_TYPE_NONE,
	SOURCE_COMPLETION_POPUP_FILTER_TYPE_TREE,
	SOURCE_COMPLETION_POPUP_FILTER_TYPE_TREE_HIDDEN
} SourceCompletionPopupFilterType;

typedef enum  {
	SOURCE_COMPLETION_POPUP_KEYS_TYPE_INFO,
	SOURCE_COMPLETION_POPUP_KEYS_TYPE_PAGE_NEXT,
	SOURCE_COMPLETION_POPUP_KEYS_TYPE_PAGE_PREV,
	SOURCE_COMPLETION_POPUP_KEYS_TYPE_LAST
} SourceCompletionPopupKeysType;


GType source_completion_popup_position_type_get_type (void);
GType source_completion_popup_filter_type_get_type (void);
GType source_completion_popup_keys_type_get_type (void);
gboolean source_completion_popup_manage_key (SourceCompletionPopup* self, const GdkEventKey* event);
void source_completion_popup_show_or_update (SourceCompletionPopup* self);
SourceCompletionPopup* source_completion_popup_construct (GType object_type);
SourceCompletionPopup* source_completion_popup_new (void);
void source_completion_popup_add_data (SourceCompletionPopup* self, SourceCompletionProposal* data);
void source_completion_popup_clear (SourceCompletionPopup* self);
gboolean source_completion_popup_select_first (SourceCompletionPopup* self);
gboolean source_completion_popup_select_last (SourceCompletionPopup* self);
gboolean source_completion_popup_select_previous (SourceCompletionPopup* self, gint rows);
gboolean source_completion_popup_select_next (SourceCompletionPopup* self, gint rows);
gboolean source_completion_popup_get_selected_proposal (SourceCompletionPopup* self, SourceCompletionProposal** proposal);
gboolean source_completion_popup_has_proposals (SourceCompletionPopup* self);
void source_completion_popup_toggle_proposal_info (SourceCompletionPopup* self);
void source_completion_popup_page_next (SourceCompletionPopup* self);
void source_completion_popup_page_previous (SourceCompletionPopup* self);
void source_completion_popup_set_current_info (SourceCompletionPopup* self, const char* info);
gint source_completion_popup_get_num_active_pages (SourceCompletionPopup* self);
void source_completion_popup_set_filter_type (SourceCompletionPopup* self, SourceCompletionPopupFilterType filter_type);
void source_completion_popup_set_filter_text (SourceCompletionPopup* self, const char* text);
char* source_completion_popup_get_filter_text (SourceCompletionPopup* self);
void source_completion_popup_set_key (SourceCompletionPopup* self, SourceCompletionPopupKeysType type, const char* keys);
char* source_completion_popup_get_key (SourceCompletionPopup* self, SourceCompletionPopupKeysType type);
SourceCompletionPopupFilterType source_completion_popup_get_filter_type (SourceCompletionPopup* self);
void source_completion_popup_bottom_bar_set_visible (SourceCompletionPopup* self, gboolean visible);
gboolean source_completion_popup_bottom_bar_get_visible (SourceCompletionPopup* self);
GType source_completion_popup_get_type (void);


G_END_DECLS

#endif
