/* utils.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __PLUGINS_COMPLETION_SOURCE_COMPLETION_UTILS_H__
#define __PLUGINS_COMPLETION_SOURCE_COMPLETION_UTILS_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>

G_BEGIN_DECLS


#define SOURCE_COMPLETION_TYPE_UTILS (source_completion_utils_get_type ())
#define SOURCE_COMPLETION_UTILS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SOURCE_COMPLETION_TYPE_UTILS, SourceCompletionUtils))
#define SOURCE_COMPLETION_UTILS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SOURCE_COMPLETION_TYPE_UTILS, SourceCompletionUtilsClass))
#define SOURCE_COMPLETION_IS_UTILS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SOURCE_COMPLETION_TYPE_UTILS))
#define SOURCE_COMPLETION_IS_UTILS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SOURCE_COMPLETION_TYPE_UTILS))
#define SOURCE_COMPLETION_UTILS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SOURCE_COMPLETION_TYPE_UTILS, SourceCompletionUtilsClass))

typedef struct _SourceCompletionUtils SourceCompletionUtils;
typedef struct _SourceCompletionUtilsClass SourceCompletionUtilsClass;
typedef struct _SourceCompletionUtilsPrivate SourceCompletionUtilsPrivate;
typedef struct _SourceCompletionParamSpecUtils SourceCompletionParamSpecUtils;

/*
 * This code is re-writing in the vala language of GtkSourceCompletion
 *
 * Copyright (C) 2007 - Chuchiperriman
 *
 * rev 163
 */
struct _SourceCompletionUtils {
	GTypeInstance parent_instance;
	volatile int ref_count;
	SourceCompletionUtilsPrivate * priv;
};

struct _SourceCompletionUtilsClass {
	GTypeClass parent_class;
	void (*finalize) (SourceCompletionUtils *self);
};

struct _SourceCompletionParamSpecUtils {
	GParamSpec parent_instance;
};


gboolean source_completion_utils_char_is_separator (gunichar ch);
char* source_completion_utils_get_last_word_and_iter (GtkTextView* text_view, GtkTextIter* start_word, GtkTextIter* end_word);
char* source_completion_utils_get_last_word (GtkTextView* text_view);
char* source_completion_utils_get_last_word_cleaned (GtkTextView* view);
void source_completion_utils_get_cursor_pos (GtkTextView* text_view, gint* x, gint* y);
char* source_completion_utils_get_text (GtkTextView* text_view);
void source_completion_utils_replace_actual_word (GtkTextView* text_view, const char* text);
char* source_completion_utils_clear_word (const char* word);
char* source_completion_utils_compute_line_indentation (GtkTextView* view, const GtkTextIter* cur);
char* source_completion_utils_get_text_with_indent (const char* content, const char* indent);
void source_completion_utils_insert_text_with_indent (GtkTextView* view, const char* text);
gboolean source_completion_utils_is_valid_word (const char* current_word, const char* completion_word);
void source_completion_utils_get_window_position_center_screen (GtkWindow* window, gint* x, gint* y);
void source_completion_utils_get_window_position_center_parent (GtkWindow* window, GtkWidget* parent, gint* x, gint* y);
gboolean source_completion_utils_get_window_position_in_cursor (GtkWindow* window, GtkTextView* view, gint* x, gint* y);
gboolean source_completion_utils_compare_keys (guint key, guint mods, const GdkEventKey* event);
SourceCompletionUtils* source_completion_utils_construct (GType object_type);
SourceCompletionUtils* source_completion_utils_new (void);
GParamSpec* source_completion_param_spec_utils (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer source_completion_value_get_utils (const GValue* value);
void source_completion_value_set_utils (GValue* value, gpointer v_object);
GType source_completion_utils_get_type (void);
gpointer source_completion_utils_ref (gpointer instance);
void source_completion_utils_unref (gpointer instance);


G_END_DECLS

#endif
