/* todo-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

#ifndef __PLUGINS_TODO_TODO_H__
#define __PLUGINS_TODO_TODO_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <libvalide/abstract-text-document.h>
#include <libvalide/plugin.h>

G_BEGIN_DECLS


#define TYPE_TODO_ITEM (todo_item_get_type ())
#define TODO_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TODO_ITEM, TodoItem))
#define TODO_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TODO_ITEM, TodoItemClass))
#define IS_TODO_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TODO_ITEM))
#define IS_TODO_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TODO_ITEM))
#define TODO_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TODO_ITEM, TodoItemClass))

typedef struct _TodoItem TodoItem;
typedef struct _TodoItemClass TodoItemClass;
typedef struct _TodoItemPrivate TodoItemPrivate;
typedef struct _ParamSpecTodoItem ParamSpecTodoItem;

#define TYPE_TODO (todo_get_type ())
#define TODO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TODO, Todo))
#define TODO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TODO, TodoClass))
#define IS_TODO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TODO))
#define IS_TODO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TODO))
#define TODO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TODO, TodoClass))

typedef struct _Todo Todo;
typedef struct _TodoClass TodoClass;
typedef struct _TodoPrivate TodoPrivate;

struct _TodoItem {
	GTypeInstance parent_instance;
	volatile int ref_count;
	TodoItemPrivate * priv;
	char* type;
	char* text;
	char* file;
	gint line;
	ValideAbstractTextDocument* document;
};

struct _TodoItemClass {
	GTypeClass parent_class;
	void (*finalize) (TodoItem *self);
};

struct _ParamSpecTodoItem {
	GParamSpec parent_instance;
};

/**
 * @todo Add support for author, like C::B
 */
struct _Todo {
	ValidePlugin parent_instance;
	TodoPrivate * priv;
};

struct _TodoClass {
	ValidePluginClass parent_class;
};


TodoItem* todo_item_construct (GType object_type);
TodoItem* todo_item_new (void);
GParamSpec* param_spec_todo_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer value_get_todo_item (const GValue* value);
void value_set_todo_item (GValue* value, gpointer v_object);
GType todo_item_get_type (void);
gpointer todo_item_ref (gpointer instance);
void todo_item_unref (gpointer instance);
Todo* todo_construct (GType object_type);
Todo* todo_new (void);
GType todo_get_type (void);
GType register_plugin (void);


G_END_DECLS

#endif
