;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NSIS installer script for vala ;
;  (http://live.gnome.org/Vala)  ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!define MINGW_DIR "e:\msys"

;;;;;;;;;;;;;;;;;;;;;;;;;
; General configuration ;
;;;;;;;;;;;;;;;;;;;;;;;;;

Name "valide-0.4"
OutFile ".\valide-0.4.exe"
InstallDir "$PROGRAMFILES\valide-0.4"
ShowInstDetails show
ShowUnInstDetails show
!ifdef NSIS_LZMA_COMPRESS_WHOLE
  SetCompressor lzma
!else
  SetCompressor /SOLID lzma
!endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NSIS Modern User Interface configuration ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; MUI 1.66 compatible
  !include "MUI.nsh"
  !include "MUI2.nsh"
  !include "EnvVarUpdate.nsh"

; MUI Settings
  !define MUI_ABORTWARNING

; Installer pages
  ; Welcome page
    !insertmacro MUI_PAGE_WELCOME
  ; License page
    !define MUI_LICENSEPAGE_RADIOBUTTONS
    !insertmacro MUI_PAGE_LICENSE "..\..\COPYING"
  ; Components page
    !insertmacro MUI_PAGE_COMPONENTS
  ; Directory page
    !insertmacro MUI_PAGE_DIRECTORY
  ; Start menu page
    Var StartMenuFolder
    !insertmacro MUI_PAGE_STARTMENU Application $StartMenuFolder
  ; Instfiles page
    !insertmacro MUI_PAGE_INSTFILES
  ; Finish page
    !define MUI_FINISHPAGE_LINK "http://code.google.com/p/valide/"
    !define MUI_FINISHPAGE_LINK_LOCATION "http://code.google.com/p/valide/"
    !insertmacro MUI_PAGE_FINISH

; Languages
  !insertmacro MUI_LANGUAGE "English"
  !insertmacro MUI_LANGUAGE "French"

; Reserve files for solid compression
  !insertmacro MUI_RESERVEFILE_LANGDLL

;;;;;;;;;;;;;;;;;;;;;;;
; Macro and Functions ;
;;;;;;;;;;;;;;;;;;;;;;;

Function .onInit
  !insertmacro MUI_LANGDLL_DISPLAY
FunctionEnd

Section "Val(a)ide" Valide
  SectionIn RO
  SetOutPath "$INSTDIR\bin"
  File "..\..\_build_\default\src\valide.exe"
  File "..\..\_build_\default\libvalide\libvalide-0.0.dll"
  File "..\..\_build_\default\ctags\ctags-vala.exe"
  SetOutPath "$INSTDIR\share\valide\plugins\completion"
  File "..\..\_build_\default\plugins\completion\libcompletion.dll"
  File "..\..\plugins\completion\completion.valide-plugin"
  SetOutPath "$INSTDIR\share\valide\plugins\completion\pixmaps"
  File "..\..\plugins\completion\pixmaps\*"
  SetOutPath "$INSTDIR\share\valide\plugins\todo"
  File "..\..\_build_\default\plugins\todo\libtodo.dll"
  File "..\..\plugins\todo\todo.valide-plugin"
  SetOutPath "$INSTDIR\share\valide\plugins\symbol"
  File "..\..\_build_\default\plugins\symbol\libsymbol.dll"
  File "..\..\plugins\symbol\symbol.valide-plugin"
  File "..\..\plugins\symbol\symbol-browser.png"
  SetOutPath "$INSTDIR\share\valide\plugins\symbol\pixmaps"
  File "..\..\plugins\symbol\pixmaps\*"
  SetOutPath "$INSTDIR\share\valide\plugins\file-browser"
  File "..\..\_build_\default\plugins\file-browser\libfile-browser.dll"
  File "..\..\plugins\file-browser\file-browser.valide-plugin"
  SetOutPath "$INSTDIR\share\valide\plugins\opened-documents"
  File "..\..\_build_\default\plugins\opened-documents\libopened-documents.dll"
  File "..\..\plugins\opened-documents\opened-documents.valide-plugin"
  SetOutPath "$INSTDIR\share\valide"
  File "..\..\COPYING"
  File "..\..\AUTHORS"
  SetOutPath "$INSTDIR\share\valide\licenses"
  File "..\..\data\licenses\*"
  SetOutPath "$INSTDIR\share\valide\template\cmd"
  File "..\..\data\template\cmd\*"
  SetOutPath "$INSTDIR\share\valide\template\empty"
  File "..\..\data\template\empty\*"
  SetOutPath "$INSTDIR\share\valide\template\gnome"
  File "..\..\data\template\gnome\*"
  SetOutPath "$INSTDIR\share\valide\template\gtk"
  File "..\..\data\template\gtk\*"
  SetOutPath "$INSTDIR\share\valide\template\sdl"
  File "..\..\data\template\sdl\*"
  SetOutPath "$INSTDIR\share\valide\template\opengl"
  File "..\..\data\template\opengl\*"
  SetOutPath "$INSTDIR\share\pixmaps\valide"
  File "..\..\pixmaps\*"
  SetOutPath "$INSTDIR\share\locale\fr\LC_MESSAGES"
  File /oname=valide.mo "..\..\_build_\default\po\fr.mo"

  ; libxml-2.0
  SetOutPath "$INSTDIR\bin"
  File "${MINGW_DIR}\bin\libxml2.dll"

  ; gtksourceview-2.0
  File "${MINGW_DIR}\bin\libgtksourceview-2.0-0.dll"
  SetOutPath "$INSTDIR\share\gtksourceview-2.0\language-specs"
  File "${MINGW_DIR}\share\gtksourceview-2.0\language-specs\*"
  SetOutPath "$INSTDIR\share\gtksourceview-2.0\styles"
  File "${MINGW_DIR}\share\gtksourceview-2.0\styles\*"

  ${EnvVarUpdate} $0 "PATH" "A" "HKLM" "$INSTDIR\bin"
SectionEnd

Section "Vala compiler" Vala
  StrCpy $2 "$TEMP\vala_Installer.exe"
  NSISdl::download http://valide.googlecode.com/files/vala-0.5.7.exe $2
  Pop $0
  StrCmp $0 success success
    MessageBox MB_OK "Download failed: $0"
    Abort
  success:
    ExecWait '"$2"'
    Delete $2
SectionEnd

LangString DESC_Valide ${LANG_ENGLISH} "IDE for vala."
LangString DESC_Valide ${LANG_FRENCH} "EDI pour vala."
LangString DESC_Vala ${LANG_ENGLISH} "Vala compiler (0.5.5)."
LangString DESC_Vala ${LANG_FRENCH} "Compilateur pour vala (0.5.5)."
!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${Valide} $(DESC_Valide)
  !insertmacro MUI_DESCRIPTION_TEXT ${Vala} $(DESC_Vala)
!insertmacro MUI_FUNCTION_DESCRIPTION_END

Section -Icons
!insertmacro MUI_STARTMENU_WRITE_BEGIN Application
  SetOutPath "$INSTDIR\bin"
  CreateDirectory "$SMPROGRAMS\$StartMenuFolder"
  CreateShortCut "$SMPROGRAMS\$StartMenuFolder\Val(a)IDE.lnk" "$INSTDIR\bin\valide.exe" "" "$INSTDIR\share\pixmaps\valide\icone.ico"
  CreateShortCut "$SMPROGRAMS\$StartMenuFolder\uninstall.lnk" "$INSTDIR\uninstall.exe"
  CreateDirectory "$DESKTOP"
  CreateShortCut "$DESKTOP\Val(a)IDE.lnk" "$INSTDIR\bin\valide.exe" "" "$INSTDIR\share\pixmaps\valide\icone.ico" "" "" "" ""
!insertmacro MUI_STARTMENU_WRITE_END
SectionEnd

Section -PostInstall
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\valide" "DisplayName" "valide"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\valide" "UninstallString" "$INSTDIR\uninstall.exe"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\valide" "Publisher" "Nicolas Joseph"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\valide" "URLInfoAbout" "http://www.valaide.org/"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\valide" "HelpLink" "http://www.valaide.org/"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\valide" "URLUpdateInfo" "http://www.valaide.org/"
  WriteUninstaller "$INSTDIR\uninstall.exe"
SectionEnd

;Section -Run
;  Exec "$INSTDIR\bin\valide.exe"
;SectionEnd

#### Uninstaller code ####

Function un.onInit
  MessageBox MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2 "Are you sure you want to completely remove $(^Name) and all of its components?" IDYES +2
  Abort
FunctionEnd

Function un.onUninstSuccess
  HideWindow
  MessageBox MB_ICONINFORMATION|MB_OK "$(^Name) was successfully removed from your computer."
FunctionEnd

Section Uninstall
  ${un.EnvVarUpdate} $0 "PATH" "R" "HKLM" "$INSTDIR\bin"
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\valide"

  Delete "$INSTDIR\uninstall.exe"
  Delete "$DESKTOP\Val(a)IDE.lnk"
  Delete "$SMPROGRAMS\$StartMenuFolder\*"
  Delete "$INSTDIR\bin\*"
  Delete "$INSTDIR\etc\gtk-2.0\*"
  Delete "$INSTDIR\lib\gtk-2.0\2.10.0\engines\*"
  Delete "$INSTDIR\share\gtksourceview-2.0\styles\*"
  Delete "$INSTDIR\share\gtksourceview-2.0\language-specs\*"
  Delete "$INSTDIR\share\pixmaps\valide\*"
  Delete "$INSTDIR\share\valide\template\opengl\*"
  Delete "$INSTDIR\share\valide\template\sdl\*"
  Delete "$INSTDIR\share\valide\template\gtk\*"
  Delete "$INSTDIR\share\valide\template\gnome\*"
  Delete "$INSTDIR\share\valide\template\empty\*"
  Delete "$INSTDIR\share\valide\template\cmd\*"
  Delete "$INSTDIR\share\valide\*"
  Delete "$INSTDIR\share\valide\plugins\symbol\pixmaps\*"
  Delete "$INSTDIR\share\valide\plugins\symbol\*"
  Delete "$INSTDIR\share\valide\plugins\todo\*"

  RMDir "$SMPROGRAMS\$StartMenuFolder"
  RMDir "$INSTDIR\share\gtksourceview-2.0\styles"
  RMDir "$INSTDIR\share\gtksourceview-2.0\language-specs"
  RMDir "$INSTDIR\share\gtksourceview-2.0"
  RMDir "$INSTDIR\share\pixmaps\valide"
  RMDir "$INSTDIR\share\pixmaps"
  RMDir "$INSTDIR\share\valide\template\opengl"
  RMDir "$INSTDIR\share\valide\template\sdl"
  RMDir "$INSTDIR\share\valide\template\gtk"
  RMDir "$INSTDIR\share\valide\template\gnome"
  RMDir "$INSTDIR\share\valide\template\empty"
  RMDir "$INSTDIR\share\valide\template\cmd"
  RMDir "$INSTDIR\share\valide\template"
  RMDir "$INSTDIR\share\valide\plugins\symbol\pixmaps"
  RMDir "$INSTDIR\share\valide\plugins\symbol"
  RMDir "$INSTDIR\share\valide\plugins\todo"
  RMDir "$INSTDIR\share\valide\plugins"
  RMDir "$INSTDIR\share\valide"
  RMDir "$INSTDIR\share"
  RMDir "$INSTDIR\etc\gtk-2.0"
  RMDir "$INSTDIR\etc"
  RMDir "$INSTDIR\lib\gtk-2.0\2.10.0\engines"
  RMDir "$INSTDIR\lib\gtk-2.0\2.10.0"
  RMDir "$INSTDIR\lib\gtk-2.0"
  RMDir "$INSTDIR\lib"
  RMDir "$INSTDIR\bin"
  RMDir "$INSTDIR"

  SetAutoClose true
SectionEnd

