/* completion.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

using Valide;

public class CompletionPlugin : Plugin
{
  private DocumentManager documents;

  private void add_completion (AbstractTextDocument document)
  {
    SourceCompletion.Manager comp = SourceCompletion.Manager.get_instance (document.text_view);

    SourceCompletion.Manager.EventOptions opts = new SourceCompletion.Manager.EventOptions ();
    opts.filter_type = SourceCompletion.Popup.FilterType.NONE;
    opts.show_bottom_bar = true;

    SourceCompletion.TriggerCustomkey ur_trigger = new SourceCompletion.TriggerCustomkey (comp, "User Request Trigger", "<Control>Return");
    ur_trigger.set_opts (opts);
    comp.register_trigger(ur_trigger);

    SourceCompletion.TriggerAutowords ac_trigger = new SourceCompletion.TriggerAutowords (comp);
    comp.register_trigger(ac_trigger);

    var prov_vala = new Completion.ValaProvider (document.text_view);
    comp.register_provider (prov_vala, SourceCompletion.TriggerAutowords.NAME);
    comp.register_provider (prov_vala, "User Request Trigger");

    var prov = new SourceCompletion.DocumentwordsProvider (document.text_view);
    comp.register_provider (prov, SourceCompletion.TriggerAutowords.NAME);
    comp.register_provider (prov, "User Request Trigger");
/*
    var prov_file = new ProviderFile (document.text_view);
    prov_file.set_file (uri);
    comp.register_provider (prov_file, "User Request Trigger");
*/
    comp.activate ();
  }

  [Callback]
  private static void on_tab_added (DocumentManager sender, AbstractTextDocument document, CompletionPlugin self)
  {
    self.add_completion (document);
  }

  [Callback]
  private static void on_tab_remove (Object sender, CompletionPlugin self)
  {
    /* Destroy CompletionInfo */
  }

  construct
  {
    this.documents = DocumentManager.get_instance ();
    Signal.connect_after (this.documents, "tab-added", (Callback)on_tab_added, this);
    Signal.connect_after (this.documents, "tab-removed", (Callback)on_tab_remove, this);

    foreach (iDocument document in this.documents.documents)
    {
      if (document is AbstractTextDocument)
      {
        this.add_completion ((AbstractTextDocument)document);
      }
    }
  }

  ~CompletionPlugin ()
  {
    SignalHandler.disconnect_by_func (this.documents, (void*)on_tab_added, this);
    SignalHandler.disconnect_by_func (this.documents, (void*)on_tab_remove, this);
  }
}


[ModuleInit]
public Type register_plugin ()
{
  return typeof (CompletionPlugin);
}

