/* trigger-autowords.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

/*
 * This code is re-writing in the vala language of GtkSourceCompletion
 *
 * Copyright (C) 2007 - Jess Barbero
 *
 * rev 163
 */

public class SourceCompletion.TriggerAutowords : Object, Trigger
{
  private const int MIN_LEN = 3;
  private const int AUTOCOMPLETION_DELAY = 200;
  public const string NAME = "GscTriggerAutowords";
  private string actual_word;
  private Gtk.TextView view;
  private uint source_id;
  private uint delay;
  private int text_offset;

  public Manager completion
  {
    get;
    construct;
  }

  private int get_text_offset ()
  {
    Gtk.TextBuffer buffer = this.view.get_buffer ();
    Gtk.TextMark mark = buffer.get_insert ();
    Gtk.TextIter iter;
    buffer.get_iter_at_mark (out iter, mark);
    return iter.get_offset ();
  }

  [Callback]
  private static bool autocompletion_key_release_cb (Gtk.Widget view,
                                                     Gdk.EventKey event,
                                                     TriggerAutowords self)
  {
    uint keyval = event.keyval;
    Manager completion = self.completion;
    if (completion != null)
    {
      if (Gdk.KeySyms.BackSpace == keyval)
      {
        /* We only actualize the completion if the popup is visible */
        if (self.completion.is_visible ())
        {
          if (self.source_id != 0)
          {
            /* Stop the event because the user is written very fast*/
            Source.remove (self.source_id);
            self.source_id = 0;
          }

          self.text_offset = self.get_text_offset ();
          /*raise event in 0,5 seconds*/
          self.source_id = Timeout.add (self.delay,
                                        self.autocompletion_raise_event);	
        }
      }
    }
    return false;
  }

  [Callback]
  private static void autocompletion_insert_text_cb (Gtk.TextBuffer buffer,
                                                     Gtk.TextIter location,
                                                     string text,
                                                     int len,
                                                     TriggerAutowords self)
  {
    /* @TODO see the maximun length of a single UTF-8 character*/
    if (len <= 2)
    {
      if (self.source_id != 0)
      {
        /* Stop the event because the user is written very fast*/
        Source.remove (self.source_id);
        self.source_id = 0;
      }

      self.text_offset = self.get_text_offset ();
      /*raise event in 0,5 seconds*/
      self.source_id = Timeout.add (self.delay, self.autocompletion_raise_event);
    }
  }

  private bool autocompletion_raise_event ()
  {
    string word;

    /* Check if the user has changed the cursor position.If yes, we don't complete */
    int offset = this.get_text_offset ();
    if (offset != this.text_offset)
	    return false;

    Manager completion = this.completion;
    Gtk.SourceView source_view = (Gtk.SourceView)this.view;
    Gtk.TextIter tmp1, tmp2;
    word = Utils.get_last_word_and_iter (source_view, out tmp1, out tmp2);
    if (word.len () >= this.MIN_LEN)
    {
	    this.actual_word = word;
	    debug ("auto");
	    completion.trigger_event (this.NAME, this);
    }
    else
    {
      Trigger active_trigger = completion.get_active_trigger ();
      if (active_trigger != null && active_trigger.get_name () == this.NAME)
      {
        completion.finish_completion ();
      }
    }
    return false;
  }

  public string get_name ()
  {
    return this.NAME;
  }

  public bool activate ()
  {
    Signal.connect_after (this.view, "key-release-event",
                          (Callback)autocompletion_key_release_cb, this);
    Signal.connect_after (this.view.buffer, "insert-text",
                          (Callback)autocompletion_insert_text_cb, this);
    return true;
  }

  public bool deactivate ()
  {
    SignalHandler.disconnect_by_func (this.view, (void*)autocompletion_key_release_cb, this);
    SignalHandler.disconnect_by_func (this.view.buffer, (void*)autocompletion_insert_text_cb, this);
    return false;
  }

  construct
  {
    this.actual_word = null;
    this.delay = this.AUTOCOMPLETION_DELAY;
    this.view = this.completion.get_view ();
    this.source_id = 0;
  }

  public TriggerAutowords (Manager completion)
  {
    this.completion = completion;
  }

  public void set_delay (uint delay)
  {
    this.delay = delay;
  }
}

