/* trigger-customkey.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <gege2061@redaction-developpez.com>
 */

/*
 * This code is re-writing in the vala language of GtkSourceCompletion
 *
 * Copyright (C) 2008 - perriman
 *
 * rev 163
 */

public class SourceCompletion.TriggerCustomkey : Trigger, Object
{
  private string trigger_name;
  private uint key;
  private Gdk.ModifierType mod;
  private Manager.EventOptions? options;

  public Manager completion
  {
    get;
    construct;
  }

  [Callbakc]
  private bool view_key_press_event_cb (Gtk.Widget view, Gdk.EventKey event)
  {
    Manager completion = this.completion;

    if (completion != null)
    {       
      uint key = Gdk.keyval_to_lower (this.key);

      if ((event.state & this.mod) ==  this.mod
          && Gdk.keyval_to_lower (event.keyval) == key)
      {
        if (this.options != null)
          completion.trigger_event_with_opts (this.trigger_name, this.options, null);
        else
          completion.trigger_event (this.trigger_name, null);
        return true;
      }
    }
    return false;
  }

  public string get_name ()
  {
    return this.trigger_name;
  }

  public bool activate ()
  {
    Manager comp = this.completion;
    Gtk.TextView view = comp.get_view ();

    assert (view != null);
    view.key_press_event += view_key_press_event_cb;
    return true;
  }

  public bool deactivate ()
  {
    Gtk.TextView view = this.completion.get_view ();
    view.key_press_event -= view_key_press_event_cb;
    return false;
  }

  construct
  {
    this.options = null;
  }

  public void set_keys (string keys)
  {
    Gtk.accelerator_parse (keys, out this.key, out this.mod);
  }

  public TriggerCustomkey (Manager completion, string trigger_name, string keys)
  {
    this.completion = completion;
    this.trigger_name = trigger_name;
    this.set_keys (keys);
  }

  public void set_opts (Manager.EventOptions? options)
  {
    /*We must clone the data?*/
    this.options = options;
  }
}

