/* utils.vala
 *
 * Copyright (C) 2008  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <gege2061@redaction-developpez.com>
 */

/*
 * This code is re-writing in the vala language of GtkSourceCompletion
 *
 * Copyright (C) 2007 - Chuchiperriman
 *
 * rev 163
 */

public class SourceCompletion.Utils
{
  public static bool char_is_separator (unichar ch)
  {
    if (ch.isprint () && (ch.isalnum () || ch == "_".get_char ()))
    {
      return false;
    }
    return true;
  }

  public static string get_last_word_and_iter (Gtk.TextView text_view, 
                                               out Gtk.TextIter start_word, 
                                               out Gtk.TextIter end_word)
  {
    Gtk.TextMark insert_mark;
    Gtk.TextBuffer text_buffer;
    Gtk.TextIter actual, temp;
    Gtk.TextIter start_iter;
    string text;
    unichar ch;
    bool found, no_doc_start;

    text_buffer = text_view.get_buffer ();
    insert_mark = text_buffer.get_insert ();
    text_buffer.get_iter_at_mark (out actual, insert_mark);

    start_iter = actual;
    end_word = actual;

    found = false;
    while ((no_doc_start = start_iter.backward_char ()) == true)
    {
      ch = start_iter.get_char ();
      //TODO Do better
      if (Utils.char_is_separator (ch))
      {
        found = true;
        break;
      }
    }

    if (!no_doc_start)
    {
      text_buffer.get_start_iter (out start_iter);
      text = start_iter.get_text (actual);
    }
    else
    {
      if (found)
      {
        start_iter.forward_char ();
        text = start_iter.get_text (actual);
      }
      else
      {
        start_iter = actual;
        text = "";
      }
    }
    start_word = start_iter;
    return text;
  }

  public static string get_last_word (Gtk.TextView text_view)
  {
    Gtk.TextIter tmp1, tmp2;
    return Utils.get_last_word_and_iter (text_view, out tmp1, out tmp2);
  }

  public static string get_last_word_cleaned (Gtk.TextView view)
  {
    Gtk.TextIter tmp1, tmp2;
    string word = Utils.get_last_word_and_iter (view, out tmp1, out tmp2);
    string clean_word = Utils.clear_word (word);
    return clean_word;
  }

  public static void get_cursor_pos (Gtk.TextView text_view, 
                                     out int x, out int y)
  {
    Gdk.Window win;
    Gtk.TextMark insert_mark;
    Gtk.TextBuffer text_buffer;
    Gtk.TextIter start;
    Gdk.Rectangle location;
    int win_x, win_y;
    int xx, yy;

    text_buffer = text_view.get_buffer ();
    insert_mark = text_buffer.get_insert ();
    text_buffer.get_iter_at_mark (out start,insert_mark);
    text_view.get_iter_location (start, out location);
    text_view.buffer_to_window_coords (Gtk.TextWindowType.WIDGET, location.x,
                                       location.y, out win_x, out win_y);

    win = text_view.get_window (Gtk.TextWindowType.WIDGET);
    win.get_origin (out xx, out yy);

    x = win_x + xx;
    y = win_y + yy + location.height;
  }

  public static string get_text (Gtk.TextView text_view)
  {
    Gtk.TextIter start, end;
    Gtk.TextBuffer buffer;

    buffer = text_view.get_buffer();
    buffer.get_start_iter (out start);
    buffer.get_end_iter (out end);
    return buffer.get_text (start, end, false);
  }

  public static void replace_actual_word (Gtk.TextView text_view, string text)
  {
    Gtk.TextBuffer buffer;
    Gtk.TextIter word_start, word_end;

    buffer = text_view.get_buffer ();
    buffer.begin_user_action ();
    Utils.get_last_word_and_iter (text_view, out word_start, out word_end);

    Gtk.TextMark mark = buffer.create_mark ("temp_replace", word_start, true);
    buffer.delete (word_start, word_end);
    buffer.get_iter_at_mark (out word_start, mark);
    buffer.insert (word_start, text, -1);
    buffer.delete_mark (mark);
    buffer.end_user_action ();
  }

  public static string? clear_word (string word)
  {
    int len = (int)word.len ();
    string temp = word;
  
    for (int i = 0; i < len; i++)
    {
      if (Utils.char_is_separator (temp.get_char ()))
        temp = temp.next_char ();
      else
        return temp;
    }
    return null;
  }

  public static string? compute_line_indentation (Gtk.TextView view,
                                                 Gtk.TextIter cur)
  {
    Gtk.TextIter start;
    Gtk.TextIter end;
    unichar ch;
    int line;

    line = cur.get_line ();

    view.get_buffer ().get_iter_at_line (out start, line);

    end = start;

    ch = end.get_char ();

    while (ch.isspace () && (ch != '\n') && (ch != '\r')
           && (end.compare (cur) < 0))
    {
      if (!end.forward_char ())
        break;

      ch = end.get_char ();
    }

    if (start.equal (end))
      return null;

    return start.get_slice (end);
  }

  public static string get_text_with_indent (string content, string indent)
  {
    StringBuilder fin = null;
    string ret = null;
    int len = (int)content.len ();
    int last_line = 0;

    for (int i = 0; i < len; i++)
    {
      if (content[i] == '\n' || content[i] == '\r')
      {
        if (fin == null)
          fin = new StringBuilder (content);
        else
        {
          fin.append_len (content.offset (last_line + 1), i - last_line);
        }
        fin.append (indent);
        last_line = i;
      }
    }
    if (fin == null)
      ret = content;
    else
    {
      if (last_line < len - 1)
      {
        fin.append_len (content.offset (last_line + 1),len - last_line);
      }
      ret = fin.str;
    }
    return ret;
  }

  public static void insert_text_with_indent (Gtk.TextView view, string text)
  {
    Gtk.TextBuffer buffer = view.get_buffer ();
    Gtk.TextMark insert = buffer.get_insert ();
    Gtk.TextIter cur;
    buffer.get_iter_at_mark (out cur, insert);
    string indent = Utils.compute_line_indentation (view, cur);
    string indent_text = Utils.get_text_with_indent (text, indent);
    buffer.insert_at_cursor (indent_text, -1);
    view.scroll_mark_onscreen (insert);
  }

  public static bool is_valid_word (string? current_word, string? completion_word)
  {
    if (completion_word == null)
      return false;
    if (current_word == null)
      return true;
  
    long len_cur = current_word.len ();
    if (current_word.collate (completion_word) == 0)
      return false;

    if (len_cur != 0 && current_word == completion_word.ndup (len_cur))
      return true;

    return false;
  }

  public static void get_window_position_center_screen (Gtk.Window window,
                                                        out int x, out int y)
  {
    int w,h;
    int sw = Gdk.Screen.width ();
    int sh = Gdk.Screen.height ();
    window.get_size (out w, out h);
    x = (sw/2) - (w/2) - 20;
    y = (sh/2) - (h/2);
  }

  public static void get_window_position_center_parent (Gtk.Window window,
                                                        Gtk.Widget parent,
                                                        out int x, out int y)
  {
    Gtk.Window parent_win = (Gtk.Window)parent.get_ancestor (typeof (Gtk.Window));
    int w, h, px, py, pw, ph;
    parent_win.get_position (out px, out py);
    parent_win.get_size (out pw, out ph);
    window.get_size (out w, out h);
  
    x = px + ((pw/2) - (w/2) -20);
    y = py + ((ph/2) - (h/2));
  }

  public static bool get_window_position_in_cursor (Gtk.Window window,
                                                    Gtk.TextView view,
                                                    out int x, out int y)
  {
    int w, h, xtext, ytext;
    int sw = Gdk.Screen.width ();
    int sh = Gdk.Screen.height ();

    Utils.get_cursor_pos (view, out x, out y);
    window.get_size (out w, out h);
    if (x + w > sw) x = sw - w - 4;
    // If we cannot show it down, we show it up.
    if (y + h > sh)
    {
      Pango.Layout layout = view.create_pango_layout (null);
      layout.get_pixel_size (out xtext, out ytext);
      y = y - ytext - h;
      return true;
    }
    return false;
  }

  public static bool compare_keys (uint key, uint mods, Gdk.EventKey event)
  {
    uint modifiers = Gtk.accelerator_get_default_mod_mask ();
    // This is for a gtk bug!!!!
    uint end_key = key;
    if ((event.state & Gdk.ModifierType.SHIFT_MASK) == Gdk.ModifierType.SHIFT_MASK)
      end_key = Gdk.keyval_to_upper (end_key);
  
    if (mods == 0 && (event.state & modifiers) == 0)
    {
      if (event.keyval == end_key)
      {
        return true;
      }
    }
    else if (((event.state & mods)  == mods) &&  event.keyval == end_key)
    {
      return true;
    }

    return false;
  }
}
